enum MOTOR {
    //% block="M1"
    1,
    //% block="M2"
    2,
    //% block="M3"
    3,
    //% block="M4"
    4
}

enum DIR {
    //% block="前进"
    FORWARD,
    //% block="后退"
    BACKWARD
}
enum COM {
    //% block="M1M2"
    1,
    //% block="M3M4"
    2
}
enum MODEL {
    //% block="全步进(单线圈)"
    SINGLE,
    //% block="全步进(双线圈)"
    DOUBLE,
    //% block="半步进"
    INTERLEAVE,
    //% block="微步进"
    MICROSTEP
}

enum SERVO {
    //% block="#0"
    0,
    //% block="#1"
    1,
    //% block="#2"
    2,
    //% block="#3"
    3,
    //% block="#4"
    4,
    //% block="#5"
    5,
    //% block="#6"
    6,
    //% block="#7"
    7
}

enum LOW_HIGH {
    //% block="HIGH"
    HIGH,
    //% block="LOW"
    LOW
}


enum PSBUTTON {
    //% block="PSB_PAD_UP"
    PSB_PAD_UP,
    //% block="PSB_PAD_DOWN"
    PSB_PAD_DOWN,
    //% block="PSB_PAD_LEFT"
    PSB_PAD_LEFT,
    //% block="PSB_PAD_RIGHT"
    PSB_PAD_RIGHT,
    //% block="PSB_TRIANGLE"
    PSB_TRIANGLE,
    //% block="PSB_CIRCLE"
    PSB_CIRCLE,
    //% block="PSB_CROSS"
    PSB_CROSS,
    //% block="PSB_SQUARE"
    PSB_SQUARE,
    //% block="PSB_L1"
    PSB_L1,
    //% block="PSB_L2"
    PSB_L2,
    //% block="PSB_L3"
    PSB_L3,
    //% block="PSB_R1"
    PSB_R1,
    //% block="PSB_R2"
    PSB_R2,
    //% block="PSB_R3"
    PSB_R3,
    //% block="PSB_SELECT"
    PSB_SELECT,
    //% block="PSB_START"
    PSB_START
}

enum PSSTK {
    //% block="PSS_RX"
    PSS_RX,
    //% block="PSS_RY"
    PSS_RY,
    //% block="PSS_LX"
    PSS_LX,
    //% block="PSS_LY"
    PSS_LY
}

enum PSBTSTAT{
     //% block="按下"
     1,
     //% block="释放"
     2,
}


//% color="#3264C8" iconWidth=50 iconHeight=40
namespace QGPMotorShield {
    export function heredoc(fn) {
        return fn.toString().split('\n').slice(1, -1).join('\n') + '\n'
    }
    //% block="初始化电机驱动" blockType="command"
    export function QGPMotorInit(parameter: any, block: any) {
        Generator.addInclude('adefine00_Wire', '#include <Wire.h>');
        Generator.addInclude('adefine02_PWMServoDriver', '#include <Adafruit_MS_PWMServoDriver.h>');
        Generator.addInclude('adefine03_MotorShield', '#include "QGPMaker_MotorShield.h"');
        
        Generator.addObject('adefine20_Motor_init', 'QGPMaker_MotorShield ', 'AFMS = QGPMaker_MotorShield();', true);

        Generator.addSetup('setup00_Motor_init_1', 'AFMS.begin(50);', true);

    }

    //% block="连接无线手柄" blockType="command"
    export function QGPPS2X_init(parameter: any, block: any) {
        Generator.addInclude('adefine01_PS2X', '#include <DFRobot_PS2X.h>');
        Generator.addObject('adefine21_PS2X_init', 'DFRobot_PS2X ', 'ps2x;', true);

        let xcode="int error = 0;do{error = ps2x.config_gamepad(13,11,10,12, true, true);if(error == 0){break;}else{delay(100);}}while(1); for(size_t i = 0; i < 50; i++){ ps2x.read_gamepad(false, 0);delay(10);}";
        Generator.addSetup('setup01_PS2X_init',xcode, true);
    }    

     //% block="接收PS2手柄数据" blockType="command"
     export function QGPPS2X_read(parameter: any, block: any) {
        Generator.addCode('ps2x.read_gamepad(false, 0);\n delay(10);');
    }   

     //% block="PS2手柄震动一次" blockType="command"
     export function QGPPS2X_vibrate(parameter: any, block: any) {
        Generator.addCode('ps2x.read_gamepad(true, 200);\ndelay(300);\nps2x.read_gamepad(false, 0);\n');
    }    
    
    //% block="PS2按钮 [button] 状态 [status]" blockType="boolean"
    //% button.shadow="dropdown" button.options="PSBUTTON" BUTTON.defl="PSBUTTON.PSB_PAD_UP"
    //% status.shadow="dropdown" status.options="PSBTSTAT" status.defl="PSBTSTAT.1"
    export function QGPPS2X_Button(parameter: any, block: any) {
        let Button = parameter.button.code;
        let BTstate = parameter.status.code;
        if (BTstate == 1) {
            Generator.addCode(`ps2x.Button(${Button})`);
        } else {
            Generator.addCode(`ps2x.ButtonReleased(${Button})`);
        }
    }

    //% block="获取PS2摇杆 [stk] 数值" blockType="reporter"
    //% stk.shadow="dropdown" stk.options="PSSTK" BUTTON.defl="PSSTK.PSS_LX"
    export function QGPPS2X_stk(parameter: any, block: any) {
        let Stick = parameter.stk.code;
        Generator.addCode(`ps2x.Analog(${Stick})`);
    }

    //% block="设置电机 [motor] 方向 [dir]速度 [speed]" blockType="command"
    //% motor.shadow="dropdown" motor.options="MOTOR" BUTTON.defl="MOTOR.M1"
    //% dir.shadow="dropdown" dir.options="DIR" dir.defl="DIR.前进"
    //% speed.shadow="range" speed.params.min=0    speed.params.max=255    speed.defl=200
    export function QGPDCMotorRun(parameter: any, block: any) {
        let Motor = parameter.motor.code;
        let Dir = parameter.dir.code;
        let Speed = parameter.speed.code;
        Generator.addObject(`adefine21_motorRun_${Motor}`, `QGPMaker_DCMotor `, `*DCMotor_${Motor} = AFMS.getMotor(${Motor});`, true);
        Generator.addCode(`DCMotor_${Motor}->setSpeed(${Speed});\n\tDCMotor_${Motor}->run(${Dir});`);
    }

    //% block="停止电机 [motor]" blockType="command"
    //% motor.shadow="dropdown" motor.options="MOTOR" BUTTON.defl="MOTOR.M1"
    export function QGPDCMotorStop(parameter: any, block: any) {
        let Motor = parameter.motor.code;
        Generator.addObject(`adefine21_motorRun_${Motor}`, `QGPMaker_DCMotor `, `*DCMotor_${Motor} = AFMS.getMotor(${Motor});`, true);
        Generator.addCode(`DCMotor_${Motor}->setSpeed(0);\n\tDCMotor_${Motor}->run(RELEASE);`);
    }


    //% block="设置舵机 [servo] 角度 [degree]" blockType="command"
    //% servo.shadow="dropdown" servo.options="SERVO" BUTTON.defl="SERVO.#1"
    //% degree.shadow="range" degree.params.min=0    degree.params.max=180    degree.defl=90
    export function QGPServoWrite(parameter: any, block: any) {

        let Servo = parameter.servo.code;
        let Degree = parameter.degree.code;
        Generator.addObject(`adefine22_Servo_${Servo}`, `QGPMaker_Servo `, `*Servo${Servo} = AFMS.getServo(${Servo});`, true);
        Generator.addCode(`Servo${Servo}->writeServo(${Degree});`);

    }

    //% block="读取舵机 [servo] 角度" blockType="reporter"
    //% servo.shadow="dropdown" servo.options="SERVO" BUTTON.defl="SERVO.#1"
    export function QGPServoRead(parameter: any, block: any) {

        let Servo = parameter.servo.code;
        Generator.addObject(`adefine22_Servo_${Servo}`, `QGPMaker_Servo `, `*Servo${Servo} = AFMS.getServo(${Servo});`, true);
        Generator.addCode(`Servo${Servo}->readDegrees()`);

    }

    //% block="PWM输出通道 [servo] PWM数值 [pwm]" blockType="command"
    //% servo.shadow="dropdown" servo.options="SERVO" BUTTON.defl="SERVO.#1"
    //% pwm.shadow="range" pwm.params.min=0    pwm.params.max=4096    pwm.defl=2048
    export function QGPPWMWrite(parameter: any, block: any) {

        let Servo = parameter.servo.code;
        let Pwm = parameter.pwm.code;
        Generator.addCode(`AFMS.setPWM(${Servo},${Pwm});`);

    }

    //% block="电子开关 通道 [servo] 状态 [pwm]" blockType="command"
    //% servo.shadow="dropdown" servo.options="SERVO" BUTTON.defl="SERVO.#1"
    //% pwm.shadow="dropdown" pwm.options="LOW_HIGH" BUTTON.defl="HIGH"
    export function QGPPINWrite(parameter: any, block: any) {

        let Servo = parameter.servo.code;
        let Pwm = parameter.pwm.code;
        Generator.addCode(`AFMS.setPin(${Servo},${Pwm});`);

    }



}
